package com.mortals.xhx.common.utils;

import cn.hutool.core.util.ReflectUtil;
import com.google.common.collect.Maps;
import com.mortals.framework.service.IUser;
import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.common.code.RuleConditionEnum;
import com.mortals.xhx.common.code.RuleTypeEnum;
import com.mortals.xhx.module.dimension.model.DimensionResourceRuleEntity;
import lombok.experimental.UtilityClass;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 权限sql 查询语句实现
 *
 * @author: zxfei
 * @date: 2024/5/15 10:44
 */
@Slf4j
@UtilityClass
public class RuleQueryGenerator {

    private static final String SQL_AND = " and ";

    private static final String SQL_OR = " or ";

    private static final String SQL_JOINT = " (%s) ";


    public static String getPermissionSql(List<DimensionResourceRuleEntity> conditionList, IUser loginUser) {
        //存在权限
        //对当前身份根据规则编码分组-去除不同角色中相同编码且规则值为ALL的规则 并根据角色id分组
        Map<Long, List<DimensionResourceRuleEntity>> ruleMap = getRuleMapByRoleId(conditionList);

        StringBuilder sb = new StringBuilder();
        String roleSql;
        if (MapUtils.isNotEmpty(ruleMap)) {
            //按角色拼接SQL
            for (Map.Entry<Long, List<DimensionResourceRuleEntity>> entry : ruleMap.entrySet()) {
                List<DimensionResourceRuleEntity> lists = entry.getValue();
                // 同角色之间使用 AND
                roleSql = buildRoleSql(lists, loginUser);
                //角色之间使用 OR
                if (StringUtils.isNotEmpty(roleSql)) {
                    jointSqlByRoles(sb, roleSql);
                }
            }

        }
        return sb.toString();
    }


    /**
     * 获取带有数据权限的SQL
     *
     * @param identityId 身份ID
     */
    public String getPermissionSql(String identityId) {
        //------------------------获取当前身份的数据规则------------------------------------
        List<DimensionResourceRuleEntity> conditionList = getCurrentIdentyPermission(identityId);
        if (ObjectUtils.isEmpty(conditionList)) {
            //没有权限
            return "1 = 0";
        }
        //存在权限
        //对当前身份根据规则编码分组-去除不同角色中相同编码且规则值为ALL的规则 并根据角色id分组
        Map<Long, List<DimensionResourceRuleEntity>> ruleMap = getRuleMapByRoleId(conditionList);

        StringBuilder sb = new StringBuilder();
        String roleSql;
        if (MapUtils.isNotEmpty(ruleMap)) {
            //按角色拼接SQL
            for (Map.Entry<Long, List<DimensionResourceRuleEntity>> entry : ruleMap.entrySet()) {

                List<DimensionResourceRuleEntity> lists = entry.getValue();

                // 同角色之间使用 AND
                roleSql = buildRoleSql(lists, null);

                //角色之间使用 OR
                if (StringUtils.isNotEmpty(roleSql)) {
                    jointSqlByRoles(sb, roleSql);
                }
            }

        }
        return sb.toString();
    }

    private static List<DimensionResourceRuleEntity> getCurrentIdentyPermission(String identityId) {
        return null;

/*        //----------------------------获取所有数据规则-----------------------------
        List<DimensionResourceRuleEntity> roleRuleList = DataPermissionUtils.loadDataSearchCondition();
        if(ObjectUtils.isEmpty(roleRuleList)){
            return null;
        }
        //-----------------------------过滤掉不属于当前身份的规则-----------------------------------
        return roleRuleList.stream()
               // .filter(item -> item.getIdentityId().equals(identityId))
                .collect(Collectors.toList());*/
    }

    /**
     * 构建单角色SQL
     */
    private static String buildRoleSql(List<DimensionResourceRuleEntity> lists, IUser loginUser) {
        StringBuilder roleSql = new StringBuilder();
        for (DimensionResourceRuleEntity item : lists) {
            //如果出现全选 则 代表全部，不需要限定范围
            if ("ALL".equals(item.getRuleValue())) {
                continue;
            }
            //将规则转换成SQL
            String filedSql = convertRuleToSql(item, loginUser);

            if(!ObjectUtils.isEmpty(filedSql)){
                roleSql.append(SQL_AND).append(filedSql);
            }

        }
        return roleSql.toString();
    }


    /**
     * 将单一规则转化成SQL，默认全部使用 In
     * ruleCode : area_test
     * ruleValue : 区域1，区域2，区域3
     *
     * @param rule 规则值
     */
    private static String convertRuleToSql(DimensionResourceRuleEntity rule, IUser loginUser) {

        //判断规则类型
        String ruleValueConvert = "";
        if (RuleConditionEnum.属于.getValue().equalsIgnoreCase(rule.getRuleCondition())) {
            String whereCondition = " in ";
            if(RuleTypeEnum.系统上下问变量.getValue()==rule.getRuleType()){
                ruleValueConvert = getInConditionValue(rule.getRuleValue(),loginUser);
                if(ObjectUtils.isEmpty(ruleValueConvert)) return null;
                return rule.getRuleCode() + whereCondition + ruleValueConvert;
            }else{
                ruleValueConvert = getInConditionValue(rule.getRuleValue());
                if(ObjectUtils.isEmpty(ruleValueConvert)) return null;
                return rule.getRuleCode() + whereCondition + ruleValueConvert;
            }
        } else if (RuleConditionEnum.大于.getValue().equalsIgnoreCase(rule.getRuleCondition())) {
            String whereCondition = " > ";
            ruleValueConvert = getGreaterConditionValue(rule.getRuleValue());
            return rule.getRuleCode() + whereCondition + ruleValueConvert;
        } else {
            //todo
            //不支持的类型
        }
        return null;

    }


    /**
     * IN字符串转换
     * 区域1, 区域2, 区域3  --> ("区域1","区域2","区域3")
     * 江西大区  --> ("江西大区")
     */
    private static String getInConditionValue(String ruleValue) {
        String[] temp = ruleValue.split(",");
        StringBuilder res = new StringBuilder();
        for (String string : temp) {
            res.append(",'").append(string).append("'");
        }
        return "(" + res.substring(1) + ")";
    }

    /**
     * IN字符串转换
     * 区域1, 区域2, 区域3  --> ("区域1","区域2","区域3")
     * 江西大区  --> ("江西大区")
     */
    private static String getInConditionValue(String ruleValue,IUser loginUser) {
        //el表达式 提取变量  再通过反射获取值，最终设置值
        Object fieldValue = ReflectUtil.getFieldValue(loginUser, ruleValue);
        if(fieldValue!=null){
            return "(" + fieldValue + ")";
        }
        return null;

    }

    /**
     * > 字符串转换
     * 区域1, 区域2, 区域3  --> ("区域1","区域2","区域3")
     * 江西大区  --> ("江西大区")
     */
    private static String getGreaterConditionValue(String ruleValue) {
        return ruleValue;
    }

    /**
     * 拼接单角色的SQL
     *
     * @param sqlBuilder 总的SQL
     * @param roleSql    单角色SQL
     */
    private static void jointSqlByRoles(StringBuilder sqlBuilder, String roleSql) {
        roleSql = roleSql.replaceFirst(SQL_AND, "");
        if (StringUtils.isEmpty(sqlBuilder.toString())) {
            sqlBuilder.append(String.format(SQL_JOINT, roleSql));
        } else {
            sqlBuilder.append(SQL_OR).append(String.format(SQL_JOINT, roleSql));
        }
    }

    /**
     * 1. 对当前身份根据规则编码分组-去除不同角色中相同编码且规则值为ALL的规则
     * 2. 对角色进行分组
     *
     * @param conditionList 数据规则
     * @return 分组后的规则list
     */
    public static Map<Long, List<DimensionResourceRuleEntity>> getRuleMapByRoleId(List<DimensionResourceRuleEntity> conditionList) {
        //--------过滤掉不属于当前身份的规则,并对条件编码进行分组-----------------------------------
        Map<String, List<DimensionResourceRuleEntity>> conditionMap = conditionList.stream().collect(Collectors.groupingBy(DimensionResourceRuleEntity::getRuleCode));

        //--------相同编码分组中存在ALL的排除掉-----------------------------------------------
        List<DimensionResourceRuleEntity> newRoleRuleList = new ArrayList<>();
        if (MapUtils.isNotEmpty(conditionMap)) {
            for (Map.Entry<String, List<DimensionResourceRuleEntity>> entry : conditionMap.entrySet()) {
                boolean flag = true;
                List<DimensionResourceRuleEntity> lists = entry.getValue();
                for (DimensionResourceRuleEntity item : lists) {
                    if ("ALL".equals(item.getRuleValue())) {
                        flag = false;
                        break;
                    }
                }

                if (flag) {
                    newRoleRuleList.addAll(lists);
                }
            }
        }
        if (CollectionUtils.isNotEmpty(newRoleRuleList)) {
            return newRoleRuleList.stream().collect(Collectors.groupingBy(DimensionResourceRuleEntity::getRoleId));
        }
        return Maps.newHashMap();
    }
}
